/* 
 * Copyright (C) 2008 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#include "boxlist.h"
#include "util.h"
#include "bittwiddler.h"

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <string.h>

void clear_box(box_t *b) {
  memset(b->symbol, 0, sizeof(b->symbol));
  b->left = 0;
  b->top = 0;
  b->right = 0;
  b->bottom = 0;
  b->page = 0;
  b->annotation = 0;
}


bool_t create_boxlist(boxlist_t *p) {
  if( p ) {
    p->isize = 128;
    p->items = (box_t *)malloc(p->isize * sizeof(box_t));
    p->count = 0;
    return (p->items != NULL);
  }
  return FALSE;
}

bool_t free_boxlist(boxlist_t *p) {
  if( p ) {
    free(p->items);
    p->isize = 0;
    p->count = 0;
  }
  return FALSE;
}

bool_t grow_boxlist(boxlist_t *p) {
  box_t *q;
  q = realloc( p->items, 2 * p->isize * sizeof(box_t));
  if( q ) {
    p->isize *= 2;
    p->items = q;
    return TRUE;
  }
  return FALSE;
}

bool_t append_boxlist(boxlist_t *p, box_t *b) {
  if( p && b) {
    if( (p->count < p->isize) || grow_boxlist(p) ) {
      memcpy(p->items + p->count, b, sizeof(box_t));
      p->count++;
      return TRUE;
    }
  }
  return FALSE;
}

bool_t insert_boxlist(boxlist_t *p, box_t *b, int pos) {
  if( p && b && (pos >= 0) ) {
    if( pos < p->count ) {
      if( append_boxlist(p, b) ) {
	memmove(p->items + pos + 1, p->items + pos, 
		 (p->count - 1 - pos) * sizeof(box_t));
	memcpy(p->items + pos, b, sizeof(box_t));
	return TRUE;
      }
    } else {
      return append_boxlist(p, b);
    }
  }
  return FALSE;
}

bool_t remove_boxlist(boxlist_t *p, int pos) {
  if( p && (pos < p->count) && (pos >= 0) ) {
    p->count--;
    memmove(p->items + pos, p->items + pos + 1, 
	    (p->count - pos) * sizeof(box_t));
    return TRUE;
  }
  return FALSE;
}

bool_t render_boxlist(boxlist_t *bl, imagebuffer_t *ib, int plane) {
  int i;
  if( bl && ib ) {
    fill_bw_imagebuffer(ib, 0, 0, ib->width, ib->height, 1, 0);
    for(i = 0; i < bl->count; i++) {
      box_bw_imagebuffer(ib, bl->items[i].left,
			 ib->height - bl->items[i].bottom,
			 bl->items[i].right,
			 ib->height - bl->items[i].top,
			 1, 1);
    }
    return TRUE;
  }
  return FALSE;
}
