/* 
 * Copyright (C) 2008 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#include "boxwriter.h"
#include "util.h"
#include "annotations.h"
#include "tessboxes.h"

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

extern options_t u_options;

bool_t create_boxwriter(boxwriter_t *p, int fd) {
  if( p ) {
    p->f = fdopen(dup(fd), "wb");
    if( p->f ) {
      return TRUE;
    }
  }
  return FALSE;
}

bool_t free_boxwriter(boxwriter_t *p) {
  if( p ) {
    if( p->f ) {
      fclose(p->f);
      p->f = NULL;
      return TRUE;
    }
  }
  return FALSE;
}


bool_t write_boxwriter(boxwriter_t *p, box_t *b, annotations_t *a) {
  if( u_options & FL_TESSV2 ) {
    /* legacy file format */
    if( 0 <= fprintf(p->f, "%s %u %u %u %u\n", b->symbol,
		     b->left, b->top, b->right, b->bottom) ) {
      return TRUE;
    }
  } else { 
    /* v3 file format */
    if( 0 <= fprintf(p->f, "%s %u %u %u %u %u #%d %s\n", b->symbol,
		     b->left, b->top, b->right, b->bottom, b->page,
		     b->annotation, get_annotations(a, b->annotation)) ) {
      return TRUE;
    }
  }
  return FALSE;
}

